@@ -28,10 +28,9 @@ __version__ = version
 DEFAULT_STORE_PATH = os.path.join(os.path.expanduser("~"), ".pod-store")
 DEFAULT_PODCAST_DOWNLOADS_PATH = os.path.join(os.path.expanduser("~"), "Podcasts")
 
-STORE_PATH = os.path.abspath(os.getenv("POD_STORE_PATH", DEFAULT_STORE_PATH))
+DO_NOT_SET_EPISODE_METADATA = os.getenv("DO_NOT_SET_POD_STORE_EPISODE_METADATA", False)
 
-STORE_FILE_NAME = os.getenv("POD_STORE_FILE_NAME", "pod-store.json")
-STORE_FILE_PATH = os.path.join(STORE_PATH, STORE_FILE_NAME)
+STORE_PATH = os.path.abspath(os.getenv("POD_STORE_PATH", DEFAULT_STORE_PATH))
 
 GPG_ID_FILE_PATH = os.getenv(
     "POD_STORE_GPG_ID_FILE", os.path.join(STORE_PATH, ".gpg-id")
@@ -42,10 +41,24 @@ try:
 except FileNotFoundError:
     GPG_ID = None
 
+if GPG_ID:
+    DEFAULT_STORE_FILE_NAME = "pod-store.gpg"
+else:
+    DEFAULT_STORE_FILE_NAME = "pod-store.json"
+
+STORE_FILE_NAME = os.getenv("POD_STORE_FILE_NAME", DEFAULT_STORE_FILE_NAME)
+STORE_FILE_PATH = os.path.join(STORE_PATH, STORE_FILE_NAME)
+
+
 STORE_GIT_REPO = os.path.join(STORE_PATH, ".git")
 
+
 PODCAST_DOWNLOADS_PATH = os.path.abspath(
     os.getenv("POD_STORE_PODCAST_DOWNLOADS_PATH", DEFAULT_PODCAST_DOWNLOADS_PATH)
 )
+PODCAST_REFRESH_TIMEOUT = float(os.getenv("POD_STORE_PODCAST_REFRESH_TIMEOUT", 15))
+EPISODE_DOWNLOAD_TIMEOUT = float(
+    os.getenv("POD_STORE_EPISODE_DOWNLOAD_TIMEOUT", PODCAST_REFRESH_TIMEOUT)
+)
 
-DO_NOT_SET_EPISODE_METADATA = os.getenv("DO_NOT_SET_POD_STORE_EPISODE_METADATA", False)
+SECURE_GIT_MODE = os.getenv("POD_STORE_SECURE_GIT_MODE", False)
