@@ -3,29 +3,25 @@ import os
 from typing import Any, List, Optional
 
 import click
+import requests
 
 from . import GPG_ID, PODCAST_DOWNLOADS_PATH, STORE_FILE_PATH, STORE_PATH
+from .commands.commit_messages import (
+    download_commit_message_builder,
+    refresh_commit_message_builder,
+    tagger_commit_message_builder,
+)
 from .commands.decorators import (
     catch_pod_store_errors,
+    conditional_confirmation_prompt,
     git_add_and_commit,
+    require_store,
     save_store_changes,
 )
-from .commands.download import download_commit_message_builder
-from .commands.helpers import (
-    abort_if_false,
-    display_pod_store_error_from_exception,
-    get_episodes,
-    get_podcasts,
-    get_tag_filters,
-)
-from .commands.ls import list_episodes_by_podcast, list_podcasts
-from .commands.refresh import refresh_commit_message_builder
-from .commands.tag import tag_commit_message_builder
-from .commands.tag_episodes import (
-    INTERACTIVE_MODE_HELP,
-    handle_episode_tagging,
-    tag_episodes_commit_message_builder,
-)
+from .commands.filtering import get_filter_from_command_arguments
+from .commands.helpers import abort_if_false, display_pod_store_error_from_exception
+from .commands.listing import get_lister_from_command_arguments
+from .commands.tagging import get_tagger_from_command_arguments
 from .store import Store
 from .store_file_handlers import EncryptedStoreFileHandler, UnencryptedStoreFileHandler
 from .util import run_git_command
@@ -117,9 +113,14 @@ def cli(ctx):
 @click.pass_context
 @click.argument("title")
 @click.argument("feed")
-@git_add_and_commit(message="Added podcast: {title!r}.", params=["title"])
-@save_store_changes
 @catch_pod_store_errors
+@require_store
+@git_add_and_commit(
+    secure_git_mode_message="Added podcast.",
+    message="Added podcast: {title!r}.",
+    params=["title"],
+)
+@save_store_changes
 def add(ctx: click.Context, title: str, feed: str):
     """Add a podcast to the store.
 
@@ -140,32 +141,95 @@ def add(ctx: click.Context, title: str, feed: str):
     help="(podcast title): Download only episodes for the specified podcast.",
 )
 @click.option(
-    "--is-tagged/--not-tagged",
-    default=True,
-    help="(flag): Search for episodes with or without the supplied tags. "
-    "Defaults to `is-tagged`. Has no effect if no tags are indicated.",
+    "-e",
+    "--episode",
+    type=int,
+    default=None,
+    help="(episode number): Download a single episode. "
+    "Note that you must specify the podcast this episode belongs to using the "
+    "`--podcast` option.",
 )
 @click.option(
-    "--tag",
+    "--tagged",
     "-t",
     multiple=True,
-    default=[],
-    help="Supply tags to search for episodes with. Multiple tags can be provided.",
+    help="Download episodes that have been tagged. Multiple tags can be provided.",
+)
+@click.option(
+    "--untagged",
+    "-u",
+    multiple=True,
+    help="Download episodes that have NOT been tagged. Multiple tags can be provided.",
+)
+@click.option(
+    "--podcast-tagged",
+    "-pt",
+    multiple=True,
+    help="Download episodes for podcasts that have been tagged. "
+    "Multiple tags can be provided.",
+)
+@click.option(
+    "--podcast-untagged",
+    "-up",
+    multiple=True,
+    help="Download episodes for podcasts that have NOT been tagged. "
+    "Multiple tags can be provided.",
 )
-@git_add_and_commit(commit_message_builder=download_commit_message_builder)
-@save_store_changes
 @catch_pod_store_errors
+@require_store
+@git_add_and_commit(
+    secure_git_mode_message="Downloaded episodes.",
+    commit_message_builder=download_commit_message_builder,
+)
+@save_store_changes
 def download(
-    ctx: click.Context, podcast: Optional[str], is_tagged: bool, tag: List[str]
+    ctx: click.Context,
+    podcast: Optional[str],
+    episode: Optional[int],
+    tagged: Optional[List[str]],
+    untagged: Optional[List[str]],
+    podcast_tagged: Optional[List[str]],
+    podcast_untagged: Optional[List[str]],
 ):
-    """Download podcast episodes."""
-    store = ctx.obj
-    tag_filters = get_tag_filters(tags=tag, is_tagged=is_tagged)
-    episodes = get_episodes(store=store, new=True, podcast_title=podcast, **tag_filters)
+    """Download podcast episodes. Use the command options to download all new episodes,
+    only new episodes for a specific podcast, only episodes with specific tags, or only
+    a single episode.
 
-    for ep in episodes:
-        click.echo(f"Downloading: {ep.download_path}")
-        ep.download()
+    By default, only new episodes will be downloaded, but when specifying a single
+    episode it will be downloaded whether or not it is marked as 'new'.
+    """
+    store = ctx.obj
+    tagged = list(tagged or [])
+    untagged = list(untagged or [])
+
+    if episode is not None:
+        new_episodes = False
+        filters = {"episode_number": episode}
+    else:
+        new_episodes = True
+        filters = {}
+
+    filter = get_filter_from_command_arguments(
+        store=store,
+        new_episodes=new_episodes,
+        filter_for_episodes=True,
+        podcast_title=podcast,
+        tagged=tagged,
+        untagged=untagged,
+        podcasts_tagged=podcast_tagged,
+        podcasts_untagged=podcast_untagged,
+        **filters,
+    )
+
+    for ep in filter.items:
+        click.echo(f"Downloading: {ep.download_path}.")
+        try:
+            with ep.download() as download:
+                with click.progressbar(download, length=download.length) as bar:
+                    for _ in bar:
+                        pass
+        except requests.ConnectTimeout:
+            click.secho("Error: timed out when downloading episode.", fg="red")
 
 
 @cli.command()
@@ -180,7 +244,11 @@ def download(
     prompt="Are you sure you want to encrypt the pod store?",
     help="Skip the confirmation prompt.",
 )
-@git_add_and_commit(message="Encrypted the store.")
+@catch_pod_store_errors
+@require_store
+@git_add_and_commit(
+    secure_git_mode_message="Set up store.", message="Encrypted the store."
+)
 def encrypt_store(ctx: click.Context, gpg_id: str):
     """Encrypt the pod store file with the provided gpg keys.
 
@@ -234,6 +302,10 @@ def init(git: bool, git_url: Optional[str], gpg_id: Optional[str]):
     flags to configure your git repo and gpg encryption.
     """
     git = git or git_url
+
+    if git_url:
+        click.echo("Please note, this could take a minute or two...")
+
     Store.init(
         store_path=STORE_PATH,
         store_file_path=STORE_FILE_PATH,
@@ -241,6 +313,7 @@ def init(git: bool, git_url: Optional[str], gpg_id: Optional[str]):
         git_url=git_url,
         gpg_id=gpg_id,
     )
+
     click.echo(f"Store created: {STORE_PATH}")
     click.echo(f"Podcast episodes will be downloaded to {PODCAST_DOWNLOADS_PATH}")
 
@@ -275,17 +348,25 @@ def init(git: bool, git_url: Optional[str], gpg_id: Optional[str]):
     help="(podcast title): List only episodes for the specified podcast.",
 )
 @click.option(
-    "--is-tagged/--not-tagged",
-    default=True,
-    help="(flag): Search for episodes with or without the supplied tags. "
-    "Defaults to `--is-tagged`. Has no effect if no tags are indicated.",
+    "-e",
+    "--episode",
+    type=int,
+    default=None,
+    help="(episode number): List a single podcast episode. "
+    "Note that you must specify a podcast this episode belongs to using the "
+    "`--podcast` option.",
 )
 @click.option(
-    "--tag",
+    "--tagged",
     "-t",
     multiple=True,
-    default=[],
-    help="Supply tags to search for episodes with. Multiple tags can be provided.",
+    help="Filter results by tag. Multiple tags can be provided.",
+)
+@click.option(
+    "--untagged",
+    "-u",
+    multiple=True,
+    help="Filter results by asence of tag. Multiple tags can be provided.",
 )
 @click.option(
     "--verbose/--not-verbose",
@@ -294,13 +375,15 @@ def init(git: bool, git_url: Optional[str], gpg_id: Optional[str]):
     "Defaults to `--not-verbose`.",
 )
 @catch_pod_store_errors
+@require_store
 def ls(
     ctx: click.Context,
     new: bool,
     episodes: bool,
     podcast: Optional[str],
-    is_tagged: bool,
-    tag: List[str],
+    episode: Optional[int],
+    tagged: Optional[List[str]],
+    untagged: Optional[List[str]],
     verbose: bool,
 ):
     """List data from the store.
@@ -309,33 +392,29 @@ def ls(
     the provided flags and command options.
     """
     store = ctx.obj
-    podcast_title = podcast
+    tagged = list(tagged or [])
+    untagged = list(untagged or [])
 
-    # Assume we are listing episodes if an individual podcast was specified.
-    list_episodes = episodes or podcast
-    tag_filters = get_tag_filters(tags=tag, is_tagged=is_tagged)
-
-    if list_episodes:
-        episode_filters = tag_filters
-        podcast_filters = {}
-        if new:
-            episode_filters["new"] = True
+    if episode is not None:
+        list_episodes = True
+        new_episodes = False
+        filters = {"episode_number": episode}
     else:
-        podcast_filters = tag_filters
+        list_episodes = bool(episodes or podcast)
+        new_episodes = new
+        filters = {}
 
-    if new:
-        podcast_filters["has_new_episodes"] = True
-
-    podcasts = get_podcasts(store=store, title=podcast_title, **podcast_filters)
-
-    if list_episodes:
-        output = list_episodes_by_podcast(
-            podcasts=podcasts, store=store, verbose=verbose, **episode_filters
-        )
-    else:
-        output = list_podcasts(podcasts, verbose=verbose)
-
-    click.echo(output)
+    lister = get_lister_from_command_arguments(
+        store=store,
+        new_episodes=new_episodes,
+        list_episodes=list_episodes,
+        podcast_title=podcast,
+        tagged=tagged,
+        untagged=untagged,
+        **filters,
+    )
+    for msg in lister.list(verbose=verbose):
+        click.echo(msg)
 
 
 @cli.command()
@@ -352,12 +431,40 @@ def ls(
     help="(flag): Run this command in interactive mode to select which episodes to "
     "mark, or bulk mode to mark all episodes. Defaults to `--interactive`.",
 )
-def mark_as_new(ctx: click.Context, podcast: Optional[str], interactive: bool):
-    """Add the `new` tag to a group of episodes. Alias for the `tag` command."""
-    # Many of the common decorators are missing from this command. This is to avoid
-    # doubling up the same decorators when we invoke the `untag_episodes` function
-    # below.
-    ctx.invoke(tag_episodes, tag="new", podcast=podcast, interactive=interactive)
+@click.option(
+    "-f",
+    "--force",
+    is_flag=True,
+    help="Skip confirmation prompt for bulk mode actions.",
+)
+@conditional_confirmation_prompt(interactive=False, override="force")
+@catch_pod_store_errors
+@require_store
+@git_add_and_commit(
+    secure_git_mode_message="Tagged items.",
+    commit_message_builder=tagger_commit_message_builder,
+    action="marked",
+    tag="new",
+)
+@save_store_changes
+def mark_as_new(
+    ctx: click.Context,
+    podcast: Optional[str],
+    interactive: bool,
+    force: Optional[bool],
+):
+    """Add the `new` tag to a group of episodes."""
+    store = ctx.obj
+    tagger = get_tagger_from_command_arguments(
+        store=store,
+        tags=["new"],
+        tag_episodes=True,
+        podcast_title=podcast,
+        action="mark",
+    )
+
+    for msg in tagger.tag_items(interactive_mode=interactive):
+        click.echo(msg)
 
 
 @cli.command()
@@ -374,23 +481,53 @@ def mark_as_new(ctx: click.Context, podcast: Optional[str], interactive: bool):
     help="(flag): Run this command in interactive mode to select which episodes to "
     "mark, or bulk mode to mark all episodes. Defaults to `--interactive`.",
 )
-def mark_as_old(ctx: click.Context, podcast: Optional[str], interactive: bool):
-    """Remove the `new` tag from a group of episodes. Alias for the `untag` command."""
-    # Many of the common decorators are missing from this command. This is to avoid
-    # doubling up the same decorators when we invoke the `untag_episodes` function
-    # below.
-    ctx.invoke(untag_episodes, tag="new", podcast=podcast, interactive=interactive)
+@click.option(
+    "-f",
+    "--force",
+    is_flag=True,
+    help="Skip confirmation prompt for bulk mode actions.",
+)
+@conditional_confirmation_prompt(interactive=False, override="force")
+@catch_pod_store_errors
+@require_store
+@git_add_and_commit(
+    secure_git_mode_message="Tagged items.",
+    commit_message_builder=tagger_commit_message_builder,
+    action="unmarked",
+    tag="new",
+)
+@save_store_changes
+def mark_as_old(
+    ctx: click.Context,
+    podcast: Optional[str],
+    interactive: bool,
+    force: Optional[bool],
+):
+    """Remove the `new` tag from a group of episodes."""
+    store = ctx.obj
+    tagger = get_tagger_from_command_arguments(
+        store=store,
+        tags=["new"],
+        tag_episodes=True,
+        podcast_title=podcast,
+        is_untagger=True,
+        action="unmark",
+    )
+
+    for msg in tagger.tag_items(interactive_mode=interactive):
+        click.echo(msg)
 
 
 @cli.command()
 @click.pass_context
 @click.argument("old")
 @click.argument("new")
+@catch_pod_store_errors
+@require_store
 @git_add_and_commit(
     message="Renamed podcast: {old!r} -> {new!r}.", params=["old", "new"]
 )
 @save_store_changes
-@catch_pod_store_errors
 def mv(ctx: click.Context, old: str, new: str):
     """Rename a podcast in the store.
 
@@ -411,37 +548,56 @@ def mv(ctx: click.Context, old: str, new: str):
     help="(podcast title): Refresh only the specified podcast.",
 )
 @click.option(
-    "--is-tagged/--not-tagged",
-    default=True,
-    help="(flag): Search for podcasts with or without the tags supplied. "
-    "Defaults to `--is-tagged`. Has no effect if no tags are indicated.",
-)
-@click.option(
-    "--tag",
+    "--tagged",
     "-t",
     multiple=True,
-    default=[],
     help="Filter podcasts by tag. Multiple tags can be provided.",
 )
-@git_add_and_commit(commit_message_builder=refresh_commit_message_builder)
-@save_store_changes
+@click.option(
+    "--untagged",
+    "-u",
+    multiple=True,
+    help="Filter podcasts by absence of tag. Multiple tags can be provided.",
+)
 @catch_pod_store_errors
+@require_store
+@git_add_and_commit(
+    secure_git_mode_message="Refreshed podcasts.",
+    commit_message_builder=refresh_commit_message_builder,
+)
+@save_store_changes
 def refresh(
-    ctx: click.Context, podcast: Optional[str], is_tagged: bool, tag: List[str]
+    ctx: click.Context,
+    podcast: Optional[str],
+    tagged: Optional[List[str]],
+    untagged: Optional[List[str]],
 ):
     """Refresh podcast episode data from the RSS feed."""
     store = ctx.obj
-    tag_filters = get_tag_filters(tags=tag, is_tagged=is_tagged)
-    podcasts = get_podcasts(store=store, title=podcast, **tag_filters)
+    tagged = list(tagged or [])
+    untagged = list(untagged or [])
 
-    for podcast in podcasts:
+    if not podcast:
+        untagged.append("inactive")
+
+    filter = get_filter_from_command_arguments(
+        store=store,
+        filter_for_episodes=False,
+        podcast_title=podcast,
+        tagged=tagged,
+        untagged=untagged,
+    )
+    for podcast in filter.items:
         click.echo(f"Refreshing {podcast.title}")
-        podcast.refresh()
+        try:
+            podcast.refresh()
+        except requests.ConnectTimeout:
+            click.secho("Error: timed out when reading RSS feed.", fg="red")
 
 
 @cli.command()
 @click.pass_context
-@click.argument("title")
+@click.argument("podcast")
 @click.option(
     "-f",
     "--force",
@@ -451,109 +607,153 @@ def refresh(
     prompt="Are you sure you want to delete this podcast?",
     help="Skip confirmation prompt.",
 )
-@git_add_and_commit(message="Removed podcast: {title!r}.", params=["title"])
-@save_store_changes
 @catch_pod_store_errors
-def rm(ctx: click.Context, title: str):
+@require_store
+@git_add_and_commit(message="Removed podcast: {podcast!r}.", params=["podcast"])
+@save_store_changes
+def rm(ctx: click.Context, podcast: str):
     """Remove a podcast from the store. This command will NOT delete the podcast
     episodes that have been downloaded.
 
-    TITLE: title of podcast to remove
+    PODCAST: title of podcast to remove
     """
     store = ctx.obj
-    store.podcasts.delete(title)
+    store.podcasts.delete(podcast)
 
 
 @cli.command()
 @click.pass_context
 @click.argument("podcast")
-@click.argument("tag")
-@click.option(
-    "-e",
-    "--episode",
-    default=None,
-    help="(episode ID): Episode to tag. "
-    "Note that this is the ID from the `ls --episodes --verbose` listing, not the "
-    "episode number.",
-)
+@catch_pod_store_errors
+@require_store
 @git_add_and_commit(
-    commit_message_builder=tag_commit_message_builder,
-    action="tagged",
+    secure_git_mode_message="Tagged items.",
+    commit_message_builder=tagger_commit_message_builder,
 )
 @save_store_changes
-@catch_pod_store_errors
-def tag(ctx: click.Context, podcast: str, tag: str, episode: Optional[str]):
-    """Tag a single podcast or episode with an arbitrary text tag. If the optional
-    episode ID is provided, it will be tagged. Otherwise, the podcast itself will
-    be tagged.
+def set_active(ctx: click.Context, podcast: str):
+    """Set a podcast to `active`. The RSS feed data be refreshed.
 
-    Note that tagging an episode requires the user to provide the podcast AND episode.
+    Reverses the `set-inactive` command.
 
-    PODCAST: title of podcast
-    TAG: arbitrary text tag
+    PODCAST: title of podcast to set as 'active'
     """
     store = ctx.obj
 
     podcast = store.podcasts.get(podcast)
-    if episode:
-        ep = podcast.episodes.get(episode)
-        ep.tag(tag)
-        click.echo(f"Tagged {podcast.title}, episode {episode} -> {tag}.")
-    else:
-        click.echo(f"Tagged {podcast.title} -> {tag}.")
-        podcast.tag(tag)
+    podcast.untag("inactive")
 
 
 @cli.command()
 @click.pass_context
-@click.argument("tag")
+@click.argument("podcast")
+@catch_pod_store_errors
+@require_store
+@git_add_and_commit(
+    secure_git_mode_message="Tagged items.",
+    commit_message_builder=tagger_commit_message_builder,
+)
+@save_store_changes
+def set_inactive(ctx: click.Context, podcast: str):
+    """Set a podcast to `inactive`. The RSS feed data will no longer be refreshed.
+
+    PODCAST: title of podcast to set as 'inactive'
+    """
+    store = ctx.obj
+
+    podcast = store.podcasts.get(podcast)
+    podcast.tag("inactive")
+
+
+@cli.command()
+@click.pass_context
+@click.option("--tag", "-t", multiple=True, required=True, help="Tags to apply.")
 @click.option(
-    "-p",
-    "--podcast",
+    "--untag",
+    "-u",
+    is_flag=True,
+    help="Remove the specified tags (rather than apply them).",
+)
+@click.option(
+    "--podcast", "-p", default=None, help="(podcast title): Tag a single podcast."
+)
+@click.option(
+    "-e",
+    "--episode",
+    type=int,
     default=None,
-    help="(podcast title): Tag episodes for only the specified podcast.",
+    help="(episode number): Tag a single podcast episode. "
+    "Note that you must specify a podcast this episode belongs to using the "
+    "`--podcast` option.",
+)
+@click.option(
+    "--episodes/--podcasts", default=True, help="Tag episodes or podcasts in groups."
 )
 @click.option(
     "--interactive/--bulk",
     default=True,
-    help="(flag): Run this command in interactive mode to select which episodes to "
-    "tag, or bulk mode to tag all episodes in the group. Defaults to `--interactive`.",
+    help="Interactively determine which items to tag, or apply the tag to all items.",
 )
+@click.option(
+    "-f",
+    "--force",
+    is_flag=True,
+    help="Skip confirmation prompt for bulk mode actions.",
+)
+@conditional_confirmation_prompt(
+    interactive=False, episode=None, podcast=None, override="force"
+)
+@catch_pod_store_errors
+@require_store
 @git_add_and_commit(
-    commit_message_builder=tag_episodes_commit_message_builder, action="tagged"
+    secure_git_mode_message="Tagged items.",
+    commit_message_builder=tagger_commit_message_builder,
 )
 @save_store_changes
-@catch_pod_store_errors
-def tag_episodes(
-    ctx: click.Context, tag: str, podcast: Optional[str], interactive: bool
+def tag(
+    ctx: click.Context,
+    tag: List[str],
+    untag: bool,
+    podcast: Optional[str],
+    episode: Optional[int],
+    episodes: bool,
+    interactive: bool,
+    force: bool,
 ):
-    """Tag episodes in groups.
+    """Tag (or untag) podcasts or episodes with arbitrary tags.
 
-    TAG: arbitrary text tag to apply
+    By default, this command works on groups (rather than individual items). Determine
+    which type of item to tag using the `--episodes` or `--podcasts` option.
+    (Defaults to episodes.)
+
+    When working in group mode, you can interactively determine which items to tag
+    using the `--interactive` option, or apply the tag to all items in the group using
+    the `--bulk` tag. (Defaults to interactive mode.)
+
+    In bulk mode, a single confirmation prompt will be shown unless the user passes in
+    the `--force` flag.
+
+    It is possible to tag individual items instead using the `--podcast` and `--episode`
+    options. Specify a podcast by title to tag the podcast. To tag an episode, specify
+    the podcast by title and then specify the episode by episode number.
     """
+    tag_episodes = bool(not podcast or episode) and bool(episodes or episode)
     store = ctx.obj
-    interactive_mode = interactive
-    podcasts = get_podcasts(store=store, title=podcast)
+    if episode is not None:
+        filters = {"episode_number": episode}
+    else:
+        filters = {}
 
-    click.echo(f"Tagging: {tag}.")
-
-    if interactive:
-        click.echo(INTERACTIVE_MODE_HELP.format(action="tag"))
-
-    for pod in podcasts:
-        for ep in pod.episodes.list(**{tag: False}):
-            # `interactive` can get switched from True -> False here, if the user
-            # decides to switch from interactive to bulk-assignment partway through
-            # the list of episodes.
-            confirmed, interactive_mode = handle_episode_tagging(
-                tag=tag,
-                action="tag",
-                interactive_mode=interactive_mode,
-                podcast=pod,
-                episode=ep,
-            )
-            if confirmed:
-                click.echo(f"Tagged {pod.title} -> [{ep.episode_number}] {ep.title}")
+    tagger = get_tagger_from_command_arguments(
+        store=store,
+        tags=tag,
+        tag_episodes=tag_episodes,
+        podcast_title=podcast,
+        is_untagger=untag,
+        filters=filters,
+    )
+    for msg in tagger.tag_items(interactive_mode=interactive):
+        click.echo(msg)
 
 
 @cli.command()
@@ -567,110 +767,18 @@ def tag_episodes(
     prompt="Are you sure you want to unencrypt the pod store?",
     help="Skip the confirmation prompt.",
 )
-@git_add_and_commit(message="Unencrypted the store.")
+@catch_pod_store_errors
+@require_store
+@git_add_and_commit(
+    secure_git_mode_message="Set up store.", message="Unencrypted the store."
+)
 def unencrypt_store(ctx: click.Context):
     """Unencrypt the pod store, saving the data in plaintext instead."""
     store = ctx.obj
-
     store.unencrypt()
     click.echo("Store was unencrypted.")
 
 
-@cli.command()
-@click.pass_context
-@click.argument("podcast")
-@click.argument("tag")
-@click.option(
-    "-e",
-    "--episode",
-    default=None,
-    help="(episode ID): Episode to untag. "
-    "Note that this is the ID from the `ls --episodes --verbose` listing, not the "
-    "episode number.",
-)
-@git_add_and_commit(
-    commit_message_builder=tag_commit_message_builder,
-    action="untagged",
-)
-@save_store_changes
-@catch_pod_store_errors
-def untag(ctx: click.Context, podcast: str, tag: str, episode: Optional[str]):
-    """Untag a single podcast or episode. If the optional episode ID is provided,
-    it will be untagged. Otherwise, the podcast itself will be untagged.
-
-    Note that untagging an episode requires the user to provide both the podcast
-    AND episode.
-
-    PODCAST: title of podcast
-    TAG: arbitrary text tag
-    """
-
-    store = ctx.obj
-
-    podcast = store.podcasts.get(podcast)
-    if episode:
-        ep = podcast.episodes.get(episode)
-        ep.untag(tag)
-        click.echo(f"Untagged {podcast.title}, episode {episode} -> {tag}.")
-    else:
-        click.echo(f"Untagged {podcast.title} -> {tag}.")
-        podcast.untag(tag)
-
-
-@cli.command()
-@click.pass_context
-@click.argument("tag")
-@click.option(
-    "-p",
-    "--podcast",
-    default=None,
-    help="(podcast title): Untag episodes for only the specified podcast.",
-)
-@click.option(
-    "--interactive/--bulk",
-    default=True,
-    help="(flag): Run this command in interactive mode to select which episodes to "
-    "untag, or bulk mode to untag all episodes in the group. Defaults to "
-    "`--interactive`.",
-)
-@git_add_and_commit(
-    commit_message_builder=tag_episodes_commit_message_builder,
-    action="untagged",
-)
-@save_store_changes
-@catch_pod_store_errors
-def untag_episodes(
-    ctx: click.Context, tag: str, podcast: Optional[str], interactive: bool
-):
-    """Untag episodes in groups.
-
-    TAG: tag to remove
-    """
-    store = ctx.obj
-    interactive_mode = interactive
-    podcasts = get_podcasts(store=store, title=podcast)
-
-    click.echo(f"Untagging: {tag}.")
-
-    if interactive:
-        click.echo(INTERACTIVE_MODE_HELP.format(action="untag"))
-
-    for pod in podcasts:
-        for ep in pod.episodes.list(**{tag: True}):
-            # `interactive` can get switched from True -> False here, if the user
-            # decides to switch from interactive to bulk-assignment partway through
-            # the list of episodes.
-            confirmed, interactive_mode = handle_episode_tagging(
-                tag=tag,
-                action="untag",
-                interactive_mode=interactive_mode,
-                podcast=pod,
-                episode=ep,
-            )
-            if confirmed:
-                click.echo(f"Untagged {pod.title} -> [{ep.episode_number}] {ep.title}")
-
-
 def main() -> None:
     """Run the Click application."""
     cli()
