@@ -2,13 +2,14 @@
 
 import functools
 import os
-from typing import Any, Callable, List
+from typing import Any, Callable, Optional
 
 import click
 
-from .. import STORE_GIT_REPO
-from ..exc import ShellCommandError
+from .. import SECURE_GIT_MODE, STORE_GIT_REPO
+from ..exc import ShellCommandError, StoreDoesNotExistError
 from ..util import run_git_command
+from .commit_messages import default_commit_message_builder
 from .helpers import display_pod_store_error_from_exception
 
 
@@ -27,37 +28,50 @@ def catch_pod_store_errors(f: Callable) -> Callable:
     return catch_pod_store_errors_inner
 
 
-def default_commit_message_builder(
-    ctx_params: dict, message: str, params: List[str] = None
-) -> str:
-    """Helper to build `git` commit messages from the Click command context.
+def conditional_confirmation_prompt(
+    override: Optional[str] = None, **conditions
+) -> Callable:
+    """Decorator for prompting the user to confirm the command if conditions are met.
 
-    `message` should be the intended `git` commit message.
+    Conditions are passed as a set of key/value pairs that represent Click command
+    params and values. If the params match the expected values, the user will be
+    prompted to confirm that they want to run this command.
 
-    If `message` is a template string, `params` acts as a list of Click context
-    param names that will be passed to the `message` template string as
-    keyword arguments.
+    The `override` argument can be set to the name of a flag param that will skip this
+    confirmation.
 
-    Example:
+    For example:
 
-        default_commit_message_builder(
-            ctx_params={"thing": "world"},
-            message="Hello {thing}.",
-            params=["thing"]
-        )
+    @conditional_confirmation_prompt(hello="world", foo="bar", override="force")
+    ...
 
-    Output:
+    Would show a prompt if both of these conditions were met:
 
-        Hello world.
+        - `hello` param had a value of 'world', `foo` param had a value of 'bar'
+        - the `force` parameter was not set to True
 
-    See the `git_add_and_commit` decorator for more information.
+    If the user does not confirm the action, the command is aborted.
     """
-    params = params or []
-    message_kwargs = {p: ctx_params[p] for p in params}
-    return message.format(**message_kwargs)
+
+    def conditional_confirmation_prompt_wrapper(f: Callable) -> Callable:
+        @functools.wraps(f)
+        def conditional_confirmation_prompt_inner(ctx, *args, **kwargs) -> Any:
+            if ctx.params.get(override) is not True:
+                for param, value in conditions.items():
+                    if ctx.params.get(param) != value:
+                        break
+                else:
+                    if click.prompt("Confirm?", type=click.Choice(["y", "n"])) != "y":
+                        raise click.Abort()
+            return f(ctx, *args, **kwargs)
+
+        return conditional_confirmation_prompt_inner
+
+    return conditional_confirmation_prompt_wrapper
 
 
 def git_add_and_commit(
+    secure_git_mode_message: str = "",
     commit_message_builder: Callable = default_commit_message_builder,
     **commit_message_builder_kwargs,
 ) -> Callable:
@@ -86,10 +100,14 @@ def git_add_and_commit(
             if not os.path.exists(STORE_GIT_REPO):
                 return resp
 
+            if SECURE_GIT_MODE:
+                commit_msg = secure_git_mode_message
+            else:
+                commit_msg = commit_message_builder(
+                    ctx_params=ctx.params, **commit_message_builder_kwargs
+                )
+
             run_git_command("add .")
-            commit_msg = commit_message_builder(
-                ctx_params=ctx.params, **commit_message_builder_kwargs
-            )
             try:
                 run_git_command(f"commit -m {commit_msg!r}")
             except ShellCommandError:
@@ -101,6 +119,24 @@ def git_add_and_commit(
     return git_add_and_commit_wrapper
 
 
+def require_store(f: Callable) -> Callable:
+    """Decorator for commands that require the pod store to be initialized.
+
+    Will raise an exception if the Click context is passed without a value assigned
+    for `ctx.obj`.
+
+    If a value has been assigned to `ctx.obj`, assume it is the pod store and carry on.
+    """
+
+    @functools.wraps(f)
+    def require_store_inner(ctx: click.Context, *args, **kwargs):
+        if not ctx.obj:
+            raise StoreDoesNotExistError()
+        return f(ctx, *args, **kwargs)
+
+    return require_store_inner
+
+
 def save_store_changes(f: Callable) -> Callable:
     """Decorator for saving changes to the store after running a command.
 
