@@ -36,5 +36,13 @@ class ShellCommandError(Exception):
     pass
 
 
+class StoreDoesNotExistError(Exception):
+    pass
+
+
 class StoreExistsError(Exception):
     pass
+
+
+class StoreIsNotEncrypted(Exception):
+    pass
